#!/usr/bin/env python
##------------------------------------------------------------------------------
##
## Name: wf3_wcs_update
##
## Use this script in a pipeline to update some WCS values in WFC3 FITS files 
## This script must run after GC and before calibration
##
## Added in error codes which are not being used
## Add OSF_DATATSET and INPATH (which is ENV. in resource file)
## Add single quotes around parameters in hedit lines
##
## mm/dd/yy PR    who        Description
## -------- ----- ---------- ---------------------------------------------------
## 01/18/07 57322 Sherbert   Get script to run in pipeline
## 04/08/07 57871 Sherbert   Get rid of IRAF with glee
##
##------------------------------------------------------------------------------

############################################################
#
# Name: irafnamefix
#
# Purpose: Replace any double-slash strings (//) in a filename
#          with a single slash (/) for stupid IRAF.
#
############################################################
def irafnamefix (filename):
    import string
    return string.replace(filename, "//", "/") 


############################################################
##
## Main program
##
############################################################
# exit codes
IMGETS_ERROR = 99
IMHEAD_ERROR = 77
WCS_ERROR = 1
WCS_OK = 0

import os, sys
import opusutil
import pyraf
import pyfits
moduleName = os.path.basename(sys.argv[0])

inpath  = os.environ["INPATH"]
dataset = os.environ["OSF_DATASET"]
root    = irafnamefix(inpath+os.sep+dataset)

msg='------------------ Started processing '+dataset+ ' ------------------ '
opusutil.PrintMsg('I',msg, moduleName)

raw = root + '_raw.fits'
spt = root + '_spt.fits'

uvis_a2_size = 2051
serial_over = 25.0
serial_over_virtual = 30.0
para_over   = 19.0
chip_xsize  = 2051.
ir_overscan = 5.0

## Open with update so that close saves the changes.
fd1=pyfits.open(raw,mode='update')
fd2=pyfits.open(spt,mode='update')
detector=fd1[0].header['DETECTOR']
subarray=fd1[0].header['SUBARRAY']
xcorner=int(fd2[1].header['XCORNER'])
ycorner=int(fd2[1].header['YCORNER'])
numrows=int(fd2[1].header['NUMROWS'])
numcols=int(fd2[1].header['NUMCOLS'])

msg='DETECTOR for this dataset is '+detector
opusutil.PrintMsg('I', msg, moduleName)         ## This could be handy during debugs

sizaxis1 = numcols
sizaxis2 = numrows

if (xcorner==0 and ycorner==0):
   cornera1 = 0
   cornera2 = 0
   if detector == 'UVIS':
      sizaxis2 = sizaxis2 / 2
else:
   if detector == 'UVIS':
      cornera1 = ycorner
      cornera2 = uvis_a2_size - xcorner - sizaxis2
      if xcorner >= uvis_a2_size:
         cornera2 = cornera2 + uvis_a2_size
   else:
      cornera1 = ycorner - ir_overscan
      cornera2 = xcorner - ir_overscan

centera1 = 1 + cornera1 + sizaxis1/2
centera2 = 1 + cornera2 + sizaxis2/2

if detector == 'UVIS':

   amp=fd1[0].header['CCDAMP']
   next=int(fd1[0].header['NEXTEND'])

   msg='NEXTEND  for this dataset is ' + str(next)
   opusutil.PrintMsg('I', msg, moduleName)

   nset = next/3
   for i in range(nset):

       ## These keywords exist only in the SCI headers
       bin=int(fd1[i*3+1].header['BINAXIS1'])
       chip=int(fd1[i*3+1].header['CCDCHIP'])

       if bin == 2:
          serial_over_virtual = serial_over_virtual - 1.0
       ltm1_1 = 1.0 / bin
       ltm2_2 = ltm1_1

       if subarray:
          overscan1 = serial_over
          overscan2 = 0.0
       else:
          if (amp=='BD') or (amp=='BC' and chip==1) or (amp=='AD' and chip==2):
              overscan1 = serial_over + 2*serial_over_virtual
              if bin == 2:
                 overscan1 = overscan1 - 2
          else:
              overscan1 = serial_over

          if chip==1:
             overscan2 = para_over
          else:
             overscan2 = 0.0

       ltv1 = (overscan1 - cornera1)/bin + (0.5 - 1.0/(2*bin))
       ltv2 = (overscan2 - cornera2)/bin + (0.5 - 1.0/(2*bin))

       print raw, "-----", "IMSET ", i+1, "-----"
       ## These keywords only in SCI extensions
       wasVal = fd1[i*3+1].header['centera1']
       fd1[i*3+1].header['centera1'] = centera1
       isVal  = fd1[i*3+1].header['centera1']
       print raw,"[",i*3+1,"],CENTERA1: " , wasVal , " -> " , isVal

       wasVal = fd1[i*3+1].header['centera2']
       fd1[i*3+1].header['centera2'] = centera2
       isVal  = fd1[i*3+1].header['centera2']
       print raw,"[",i*3+1,"],CENTERA2: " , wasVal , " -> " , isVal

       wasVal = fd1[i*3+1].header['sizaxis1']
       fd1[i*3+1].header['sizaxis1'] = sizaxis1
       isVal  = fd1[i*3+1].header['sizaxis1']
       print raw,"[",i*3+1,"],SIZAXIS1: " , wasVal , " -> " , isVal

       wasVal = fd1[i*3+1].header['sizaxis2']
       fd1[i*3+1].header['sizaxis2'] = sizaxis2
       isVal  = fd1[i*3+1].header['sizaxis2']
       print raw,"[",i*3+1,"],SIZAXIS2: " , wasVal , " -> " , isVal

       for j in range(3):

           ## These keywords in all UVIS extensions
           wasVal = fd1[i*3+1+j].header['ltm1_1']
           fd1[i*3+1+j].header['ltm1_1'] = ltm1_1
           isVal  = fd1[i*3+1+j].header['ltm1_1']
           print raw,"[",i*3+1+j,"],LTM1_1  : " , wasVal , " -> " , isVal

           wasVal = fd1[i*3+1+j].header['ltm2_2']
           fd1[i*3+1+j].header['ltm2_2'] = ltm2_2
           isVal  = fd1[i*3+1+j].header['ltm2_2']
           print raw,"[",i*3+1+j,"],LTM2_2  : " , wasVal , " -> " , isVal

           wasVal = fd1[i*3+1+j].header['ltv1']
           fd1[i*3+1+j].header['ltv1'] = ltv1
           isVal  = fd1[i*3+1+j].header['ltv1']
           print raw,"[",i*3+1+j,"],LTV1    : " , wasVal , " -> " , isVal

           wasVal = fd1[i*3+1+j].header['ltv2']
           fd1[i*3+1+j].header['ltv2'] = ltv2
           isVal  = fd1[i*3+1+j].header['ltv2']
           print raw,"[",i*3+1+j,"],LTV2    : " , wasVal , " -> " , isVal

else:

   overscan1 = ir_overscan
   overscan2 = ir_overscan
   ltv1 = overscan1 - cornera1
   ltv2 = overscan2 - cornera2
   ltm1_1 = 1.0
   ltm2_2 = 1.0

   next=int(fd1[0].header['NEXTEND'])

   msg='NEXTEND  for this dataset is ' + str(next)
   opusutil.PrintMsg('I', msg, moduleName)

   nset = next/5
   for i in range(nset):

       print raw, "-----", "IMSET ", i+1, "-----"
       ## These keywords only in SCI extensions
       wasVal = fd1[i*5+1].header['centera1']
       fd1[i*5+1].header['centera1'] = centera1
       isVal  = fd1[i*5+1].header['centera1']
       print raw,"[",i*5+1,"],CENTERA1: " , wasVal , " -> " , isVal

       wasVal = fd1[i*5+1].header['centera2']
       fd1[i*5+1].header['centera2'] = centera2
       isVal  = fd1[i*5+1].header['centera2']
       print raw,"[",i*5+1,"],CENTERA2: " , wasVal , " -> " , isVal

       wasVal = fd1[i*5+1].header['sizaxis1']
       fd1[i*5+1].header['sizaxis1'] = sizaxis1
       isVal  = fd1[i*5+1].header['sizaxis1']
       print raw,"[",i*5+1,"],SIZAXIS1: " , wasVal , " -> " , isVal

       wasVal = fd1[i*5+1].header['sizaxis2']
       fd1[i*5+1].header['sizaxis2'] = sizaxis2
       isVal  = fd1[i*5+1].header['sizaxis2']
       print raw,"[",i*5+1,"],SIZAXIS2: " , wasVal , " -> " , isVal

       for j in range(3):

           ## These keywords only in SCI, ERR, and DQ (not TIME, SAMP) extensions
           wasVal = fd1[i*5+1+j].header['ltm1_1']
           fd1[i*5+1+j].header['ltm1_1'] = ltm1_1
           isVal  = fd1[i*5+1+j].header['ltm1_1']
           print raw,"[",i*5+1+j,"],LTM1_1  : " , wasVal , " -> " , isVal

           wasVal = fd1[i*5+1+j].header['ltm2_2']
           fd1[i*5+1+j].header['ltm2_2'] = ltm2_2
           isVal  = fd1[i*5+1+j].header['ltm2_2']
           print raw,"[",i*5+1+j,"],LTM2_2  : " , wasVal , " -> " , isVal

           wasVal = fd1[i*5+1+j].header['ltv1']
           fd1[i*5+1+j].header['ltv1'] = ltv1
           isVal  = fd1[i*5+1+j].header['ltv1']
           print raw,"[",i*5+1+j,"],LTV1    : " , wasVal , " -> " , isVal

           wasVal = fd1[i*5+1+j].header['ltv2']
           fd1[i*5+1+j].header['ltv2'] = ltv2
           isVal  = fd1[i*5+1+j].header['ltv2']
           print raw,"[",i*5+1+j,"],LTV2    : " , wasVal , " -> " , isVal

fd1.close(output_verify='ignore')
fd2.close()

msg='------------------ Completed processing '+dataset+ ' ------------------ '
opusutil.PrintMsg('I',msg, moduleName)
sys.exit(WCS_OK)
